@@ -1,10 +1,10 @@
-#[cfg(not(any(target_arch = "x86", target_arch = "x86_64")))]
+#[cfg(not(all(any(target_arch = "x86", target_arch = "x86_64"), not(feature = "disable_simd"))))]
 #[inline(always)]
 pub fn as_i32(value: f32) -> i32 {
     value as i32
 }
 
-#[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
+#[cfg(all(any(target_arch = "x86", target_arch = "x86_64"), not(feature = "disable_simd")))]
 #[inline(always)]
 pub fn as_i32(value: f32) -> i32 {
     #[cfg(target_arch = "x86")]
